<?php
/**
 * Attributes.
 *
 * @package iconic-woothumbs
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * Iconic_WooThumbs_Images.
 */
class Iconic_WooThumbs_Attributes {
	/**
	 * Run.
	 */
	public static function run() {
		add_action( 'woocommerce_before_variations_form', array( __CLASS__, 'generate_attributes_data' ), 10 );
	}

	/**
	 * Output JSON data for attribute images.
	 */
	public static function generate_attributes_data() {
		global $product;

		if ( ! $product || ! $product->is_type( 'variable' ) ) {
			return;
		}

		$attributes_data = get_post_meta( $product->get_id(), '_iconic_woothumbs_ai', true );

		if ( ! $attributes_data ) {
			return;
		}

		$attributes_images = array();

		foreach ( $attributes_data as $attribute_name => $attribute_data ) {
			$attributes_images[ $attribute_name ] = array();

			if ( ! $attribute_data['terms'] ) {
				continue;
			}

			foreach ( $attribute_data['terms'] as $term_data ) {
				if ( ! $term_data['term'] || ! $term_data['images'] ) {
					continue;
				}

				$image_ids = array_filter( array_column( $term_data['images'], 'id' ) );

				if ( ! $image_ids ) {
					continue;
				}

				$images = Iconic_WooThumbs_Product::get_ai_images( $product->get_id(), $image_ids );

				if ( ! $images ) {
					continue;
				}

				$attributes_images[ $attribute_name ][ sanitize_title( $term_data['term'] ) ] = $images;
			}
		}
		?>
		<script>
			(function( $, document ) {
				window.iconic_woothumbs_attributes_data = <?php echo wp_json_encode( $attributes_images ); ?>;
			}( jQuery, document ));
		</script>
		<?php
	}
}
